local settings=
{
	boardSizeMod=0,
	boardSizeMin=3,
	boardSizeMax=11,
	boardSizeRandom=false,
	boardSizeRandomMin=0,
	boardSizeRandomMax=0,
	symbolComplexityMod=0,
	bufSizeMod=0.0,
	timeScaleMod=1.0,
	--trapScaleMod=1.0,
	commandLengthMod=0,
	commandLengthMin=2,
	commandLengthMax=5,
	commandLengthRandom=false,
	commandLengthRandomMin=0,
	commandLengthRandomMax=0,
	forceUnlockMod=false,
	forceUnlockPreDefMod=false
}

local GameUI = require('GameUI')
local GameSettings = require('GameSettings')
local lang = ""

registerForEvent("onInit", function()
	lang = NameToString(GameSettings.Get("/language/OnScreen"))

	SetupLanguageListener()
	LoadSettings()
	SetupMenu()

	Observe("HackingMinigameGameController", "GetBreachingBreachedData", function (_, data)
		data.boardSizeMod=settings.boardSizeMod
		data.boardSizeMin=settings.boardSizeMin
		data.boardSizeMax=settings.boardSizeMax
		data.boardSizeRandom=settings.boardSizeRandom
		data.boardSizeRandomMin=settings.boardSizeRandomMin
		data.boardSizeRandomMax=settings.boardSizeRandomMax
		data.symbolComplexityMod=settings.symbolComplexityMod
		data.bufSizeMod=settings.bufSizeMod
		data.timeScaleMod=settings.timeScaleMod
		--data.trapScaleMod=settings.trapScaleMod
		data.commandLengthMod=settings.commandLengthMod
		data.commandLengthMin=settings.commandLengthMin
		data.commandLengthMax=settings.commandLengthMax
		data.commandLengthRandom=settings.commandLengthRandom
		data.commandLengthRandomMin=settings.commandLengthRandomMin
		data.commandLengthRandomMax=settings.commandLengthRandomMax
		data.forceUnlockMod=settings.forceUnlockMod
		data.forceUnlockPreDefMod=settings.forceUnlockPreDefMod
	end)
end)

function SetupLanguageListener()
	GameUI.Listen("MenuNav", function(state)
		if state.lastSubmenu ~= nil and state.lastSubmenu == "Settings" then
			local newLang = NameToString(GameSettings.Get("/language/OnScreen"))
			if lang ~= newLang then
				lang = newLang
				SetupMenu()
			end
			SaveSettings()
		end
	end)
end

function LoadSettings()
	local file = io.open('settings.json', 'r')
	if file ~= nil then
		local contents = file:read("*a")
		local validJson, savedSettings = pcall(function() return json.decode(contents) end)
		file:close()

		if validJson then
			for key, _ in pairs(settings) do
				if savedSettings[key] ~= nil then
					settings[key] = savedSettings[key]
				end
			end
		end
	end
end

function SaveSettings()
	local validJson, contents = pcall(function() return json.encode(settings) end)

	if validJson and contents ~= nil then
		local file = io.open("settings.json", "w+")
		file:write(contents)
		file:close()
	end
end

function SetupMenu()
	local nativeSettings = GetMod("nativeSettings")

	if not nativeSettings then
		return
	end

	if not nativeSettings.pathExists("/BreachingBreached") then
		--nativeSettings.addTab("/BreachingBreached", "Breaching")
		nativeSettings.addTab("/BreachingBreached", GetModName())
	end

	if nativeSettings.pathExists("/BreachingBreached/breaching_breached") then
		nativeSettings.removeSubcategory("/BreachingBreached/breaching_breached")
	end
	nativeSettings.addSubcategory("/BreachingBreached/breaching_breached", GetCategoryName())

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetBoardSizeModName(), GetBoardSizeModDesc(), -2, 6, 1, settings.boardSizeMod, 0, function(value)
		settings.boardSizeMod = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetBoardSizeMinName(), GetBoardSizeMinDesc(), 3, 8, 1, settings.boardSizeMin, 5, function(value)
		settings.boardSizeMin = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetBoardSizeMaxName(), GetBoardSizeMaxDesc(), 6, 11, 1, settings.boardSizeMax, 7, function(value)
		settings.boardSizeMax = value
	end)

	nativeSettings.addSwitch("/BreachingBreached/breaching_breached", GetBoardSizeRandomName(), GetBoardSizeRandomDesc(), settings.boardSizeRandom, false, function(value)
		settings.boardSizeRandom = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetBoardSizeRandomMinName(), GetBoardSizeRandomMinDesc(), 5, 9, 1, settings.boardSizeRandomMin, 5, function(value)
		settings.boardSizeRandomMin = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetBoardSizeRandomMaxName(), GetBoardSizeRandomMaxDesc(), 6, 11, 1, settings.boardSizeRandomMax, 7, function(value)
		settings.boardSizeRandomMax = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetSymbolsSizeModName(), GetSymbolsSizeModDesc(), -2, 2, 1, settings.symbolComplexityMod, 0, function(value)
		settings.symbolComplexityMod = value
	end)

	nativeSettings.addRangeFloat("/BreachingBreached/breaching_breached", GetBufSizeModName(), GetBufSizeModDesc(), -3, 5, 1, "%.f", settings.bufSizeMod, 0, function(value)
		settings.bufSizeMod = value
	end)

	nativeSettings.addRangeFloat("/BreachingBreached/breaching_breached", GetTimeScaleModName(), GetTimeScaleModDesc(), 0.00, 10, 0.05, "%.2f", settings.timeScaleMod, 1, function(value)
		settings.timeScaleMod = value
	end)

--[[ Traps do not appear to be included in the game at the moment, so this is commented out for now.
	nativeSettings.addRangeFloat("/BreachingBreached/breaching_breached", GetTrapScaleModName(), GetTrapScaleModDesc(), 0.05, 5, 0.05, "%.2f", settings.trapScaleMod, 1, function(value)
		settings.trapScaleMod = value
	end)
]]--

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetCommandLengthModName(), GetCommandLengthModDesc(), -2, 4, 1, settings.commandLengthMod, 0, function(value)
		settings.commandLengthMod = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetCommandLengthMinName(), GetCommandLengthMinDesc(), 1, 5, 1, settings.commandLengthMin, 2, function(value)
		settings.commandLengthMin = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetCommandLengthMaxName(), GetCommandLengthMaxDesc(), 3, 7, 1, settings.commandLengthMax, 5, function(value)
		settings.commandLengthMax = value
	end)

	nativeSettings.addSwitch("/BreachingBreached/breaching_breached", GetCommandLengthRandomName(), GetCommandLengthRandomDesc(), settings.commandLengthRandom, false, function(value)
		settings.commandLengthRandom = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetCommandLengthRandomMinName(), GetCommandLengthRandomMinDesc(), 2, 5, 1, settings.commandLengthRandomMin, 2, function(value)
		settings.commandLengthRandomMin = value
	end)

	nativeSettings.addRangeInt("/BreachingBreached/breaching_breached", GetCommandLengthRandomMaxName(), GetCommandLengthRandomMaxDesc(), 3, 7, 1, settings.commandLengthRandomMax, 5, function(value)
		settings.commandLengthRandomMax = value
	end)

	nativeSettings.addSwitch("/BreachingBreached/breaching_breached", GetForceUnlockModName(), GetForceUnlockModDesc(), settings.forceUnlockMod, false, function(value)
		settings.forceUnlockMod = value
	end)

	nativeSettings.addSwitch("/BreachingBreached/breaching_breached", GetForceUnlockPreDefModName(), GetForceUnlockPreDefModDesc(), settings.forceUnlockPreDefMod, false, function(value)
		settings.forceUnlockPreDefMod = value
	end)

	nativeSettings.refresh()
end

function GetModName()
	if lang == 'fr-fr' then return "Piratage"
	else return "Breaching"
	end
end

function GetCategoryName()
	if lang == 'fr-fr' then return "Protocole de piratage"
	else return "Breaching Breached Mod"
	end
end

function GetBoardSizeModName()
	if lang == 'fr-fr' then return "Modification de la taille du tableau"
	else return "Board Size Mod"
	end
end

function GetBoardSizeModDesc()
	if lang == 'fr-fr' then return "Modifie la taille du tableau du mini-jeu. Le jeu utilise 5, 6 et 7 selon le niveau de difficulté ; la valeur indiqué est ajoutée à ce nombre. Les valeurs min/max ci-dessous le bloquent dans une plage spécifique. Une valeur de 0 signifie l'utilisation des paramètres de base."
	else return "Alters the size of the minigame board. The game uses 5, 6 and 7 depending on the difficulty level; the mod value is added to this number. The min/max values below lock it into a specific range. A value of 0 means the use of vanilla settings."
	end
end

function GetBoardSizeMinName()
	if lang == 'fr-fr' then return "Limite inférieure de la taille du tableau"
	else return "Board Size Lower Limit"
	end
end

function GetBoardSizeMinDesc()
	if lang == 'fr-fr' then return "Applique une limite inférieure à la taille du tableau de mini-jeu afin qu'il ne puisse pas aller en dessous de cette valeur. La limite inférieure originale est de 5. Gardez à l'esprit que des valeurs trop petites ne vous permettront pas d'obtenir certaines (ou toutes) les récompenses."
	else return "Applies a lower cap to the size of the minigame board so it can't fall below this value. The vanilla lower limit is 5. Keep in mind too small values won't allow you to pick up some (or all) rewards."
	end
end

function GetBoardSizeMaxName()
	if lang == 'fr-fr' then return "Limite supérieure de la taille du tableau"
	else return "Board Size Upper Limit"
	end
end

function GetBoardSizeMaxDesc()
	if lang == 'fr-fr' then return "Applique une limite supérieure à la taille du tableau de mini-jeu afin qu'il ne puisse pas aller en dessus de cette valeur. La limite supérieure originale est de 7. La limite inférieure a la priorité sur la limite supérieure en cas de chevauchement des valeurs."
	else return "Applies an upper cap to the size of the minigame board so it can't go above this value. The vanilla upper limit is 7. The lower limit takes precedence over the upper limit in case the values overlap."
	end
end

function GetBoardSizeRandomName()
	if lang == 'fr-fr' then return "Taille aléatoire du tableau"
	else return "Randomize Board Size"
	end
end

function GetBoardSizeRandomDesc()
	if lang == 'fr-fr' then return "Modifie la taille du tableau de mini-jeu en utilisant une valeur aléatoire entre les valeurs min/max ci-dessous, en remplaçant entièrement la valeur interne (au lieu d'ajouter une valeur fixe via la 'Taille du tableau' à la valeur interne). La taille aléatoire ne peut jamais être inférieure à 3 ou supérieure à 11, quelles que soient les valeurs min/max ci-dessous."
	else return "Alters the size of the minigame board using a random value between the min/max values below, replacing the internal value entirely (instead of adding a fixed value via the 'Board Size Mod' to the internal value). The randomed size can never be less than 3 or higher than 11, no matter what the min/max values below are."
	end
end

function GetBoardSizeRandomMinName()
	if lang == 'fr-fr' then return "Limite inférieure pour la taille aléatoire du tableau"
	else return "Lower limit for Board Size Randomization"
	end
end

function GetBoardSizeRandomMinDesc()
	if lang == 'fr-fr' then return "La valeur aléatoire ne sera jamais inférieur à cela. Doit être inférieur à la 'Limite supérieur pour la taille aléatoire du tableau', sinon ça n'aura aucun effet."
	else return "The random value will never be lower than this. Must be lower than 'Upper limit for Board Size Randomization' or else there's no effect."
	end
end

function GetBoardSizeRandomMaxName()
	if lang == 'fr-fr' then return "Limite supérieur pour la taille aléatoire du tableau"
	else return "Upper limit for Board Size Randomization"
	end
end

function GetBoardSizeRandomMaxDesc()
	if lang == 'fr-fr' then return "La valeur aléatoire ne sera jamais supérieur à cela. Doit être supérieur à la 'Limite inférieure pour la taille aléatoire du tableau', sinon ça n'aura aucun effet."
	else return "The random value will never be higher than this. Must be higher than 'Lower limit for Board Size Randomization' or else there's no effect."
	end
end

function GetSymbolsSizeModName()
	if lang == 'fr-fr' then return "Complexité des symboles"
	else return "Symbol Complexity"
	end
end

function GetSymbolsSizeModDesc()
	if lang == 'fr-fr' then return "Modifie la complexité des symboles dans le mini-jeu. Il n'y a que trois niveaux de complexité, donc vous ne pouvez monter et descendre que de 2 au mieux. Une validation garantit que votre valeur moddée ne bloque pas le jeu. Une valeur de 0 signifie que vous utilisez les paramètres de base."
	else return "Alters symbol complexity in the minigame. There are only three levels of complexity, so you can go up and down only 2 at best. A validation ensures your modded value does not break the game. A value of 0 means using the vanilla settings."
	end
end

function GetBufSizeModName()
	if lang == 'fr-fr' then return "Modification de la taille du tampon"
	else return "Buffer Size Mod"
	end
end

function GetBufSizeModDesc()
	if lang == 'fr-fr' then return "Ajoute ou supprime des emplacements tampon dans le mini-jeu. Gardez à l'esprit que des valeurs trop faibles ne vous permettront pas de récupérer certaines (ou toutes) les récompenses. Une valeur de 0 signifie l'utilisation des paramètres de base. La taille minimale du tampon est de 4"
	else return "Adds or removes buffer slots in the minigame. Keep in mind too little values won't allow you to pick up some (or all) rewards. A value of 0 means the use of vanilla settings. The minimum buffer size is 4."
	end
end

function GetTimeScaleModName()
	if lang == 'fr-fr' then return "Modificateur du minuteur"
	else return "Timer Scale"
	end
end

function GetTimeScaleModDesc()
	if lang == 'fr-fr' then return "Applique un multiplicateur fixe à la limite de temps du mini-jeu. Une valeur de 0.0 signifie que le minuteur est désactivé. Une valeur de 1.0 signifie l'utilisation des paramètres de base."
	else return "Applies a fixed multiplier to the time limit of the minigame. A value of 0.0 means the timer is disabled. A value of 1.0 means the use of vanilla settings."
	end
end

--[[ Traps do not appear to be included in the game at the moment, so this is commented out for now.
function GetTrapScaleModName()
	if lang == 'fr-fr' then return "Probabilité des pièges"
	else return "Trap Probability Scale"
	end
end

function GetTrapScaleModDesc()
	if lang == 'fr-fr' then return "Applique un multiplicateur fixe à la probabilité des pièges dans le mini-jeu. Cette valeur ne peut être nulle pour éviter les erreurs DIV/0. Une valeur de 1.0 signifie l'utilisation des paramètres de base."
	else return "Applies a fixed multiplier to the probability of traps in the minigame. This value cannot be zero to prevent DIV/0 errors. A value of 1.0 means the use of vanilla settings."
	end
end
]]--

function GetCommandLengthModName()
	if lang == 'fr-fr' then return "Modification de la longueur des commandes"
	else return "Command Length Mod"
	end
end

function GetCommandLengthModDesc()
	if lang == 'fr-fr' then return "Raccourcit ou allonge la longueur de chaque commande dans le mini-jeu. Gardez à l'esprit que des valeurs trop faibles ne seront pas stimulantes, des valeurs trop élevées ne vous permettront pas d'obtenir certaines (ou toutes) les récompenses. Une valeur de 0 signifie l'utilisation des paramètres de base."
	else return "Shortens or extends the length of each command in the minigame. Keep in mind too little values will be unchallenging, too high values won't allow you to pick up some (or all) rewards. A value of 0 means the use of vanilla settings."
	end
end

function GetCommandLengthMinName()
	if lang == 'fr-fr' then return "Limite inférieure des commandes"
	else return "Command Length Lower Limit"
	end
end

function GetCommandLengthMinDesc()
	if lang == 'fr-fr' then return "Applique une limite inférieure à la longueur de chaque commande dans le mini-jeu afin qu'elle ne puisse pas tomber en dessous de cette valeur. La limite inférieure de base est de 2"
	else return "Applies a lower cap to the length of each command in the minigame so it can't fall below this value. The vanilla lower limit is 2."
	end
end

function GetCommandLengthMaxName()
	if lang == 'fr-fr' then return "Limite supérieure des commandes"
	else return "Command Length Upper Limit"
	end
end

function GetCommandLengthMaxDesc()
	if lang == 'fr-fr' then return "Applique une limite supérieure à la longueur de chaque commande dans le mini-jeu afin qu'elle ne puisse pas dépasser cette valeur. La limite supérieure de base est de 5 ou la taille du tampon. Gardez à l'esprit que des valeurs trop élevées ne vous permettront pas de récupérer certaines (ou toutes) les récompenses. La limite inférieure est prioritaire sur la limite supérieure en cas de chevauchement des valeurs."
	else return "Applies an upper cap to the length of each command in the minigame so it can't go above this value. The vanilla upper limit is 5 or the buffer size. Keep in mind too high values won't allow you to pick up some (or all) rewards. The lower limit takes precedence over the upper limit in case the values overlap."
	end
end

function GetCommandLengthRandomName()
	if lang == 'fr-fr' then return "Longueur de commande aléatoire"
	else return "Randomize Command Length"
	end
end

function GetCommandLengthRandomDesc()
	if lang == 'fr-fr' then return "Modifie la longueur de la commande en utilisant une valeur aléatoire entre les valeurs min/max ci-dessous, en remplaçant entièrement la valeur interne (au lieu d'ajouter une valeur fixe via la 'Longueur des commandes' à la valeur interne). La taille aléatoire ne peut jamais être inférieure à 2 ou supérieure à la taille de votre tampon, quelles que soient les valeurs min/max ci-dessous."
	else return "Alters the command length using a random value between the min/max values below, replacing the internal value entirely (instead of adding a fixed value via the 'Command Length Mod' to the internal value). The randomed size can never be less than 2 or higher than your buffer size, no matter what the min/max values below are."
	end
end

function GetCommandLengthRandomMinName()
	if lang == 'fr-fr' then return "Limite inférieure pour la longueur de commande aléatoire"
	else return "Lower limit for Command Length Randomization"
	end
end

function GetCommandLengthRandomMinDesc()
	if lang == 'fr-fr' then return "La valeur aléatoire ne sera jamais inférieur à cela. Doit être inférieur à la 'Limite supérieur pour la longueur de commande aléatoire', sinon ça n'aura aucun effet."
	else return "The random value will never be lower than this. Must be lower than 'Upper limit for Command Length Randomization' or else there's no effect."
	end
end

function GetCommandLengthRandomMaxName()
	if lang == 'fr-fr' then return "Limite supérieur pour la longueur de commande aléatoire"
	else return "Upper limit for Command Length Randomization"
	end
end

function GetCommandLengthRandomMaxDesc()
	if lang == 'fr-fr' then return "La valeur aléatoire ne sera jamais supérieure à cela. Doit être supérieur à la 'Limite inférieure pour la longueur de commande aléatoire', sinon ça n'aura aucun effet."
	else return "The random value will never be higher than this. Must be higher than 'Lower limit for Command Length Randomization' or else there's no effect."
	end
end

function GetForceUnlockModName()
	if lang == 'fr-fr' then return "Déblocage forcé"
	else return "Force Unlock"
	end
end

function GetForceUnlockModDesc()
	if lang == 'fr-fr' then return "Force le déblocage de toutes les récompenses du mini-jeu, ce qui permet de terminer le mini-jeu en sélectionnant n'importe quelle cellule du tableau."
	else return "Forces unlocking all minigame rewards, which allows completing the minigame with selecting any single cell on the board."
	end
end

function GetForceUnlockPreDefModName()
	if lang == 'fr-fr' then return "Forcer le déverrouillage de mini-jeux prédéfinis"
	else return "Force Unlock Predefined Minigames"
	end
end

function GetForceUnlockPreDefModDesc()
	if lang == 'fr-fr' then return "Force le déverrouillage de toutes les récompenses du mini-jeu dans des jeux prédéfinis, ce qui permet de terminer le mini-jeu en sélectionnant n'importe quelle cellule du tableau."
	else return "Forces unlocking all minigame rewards in predefined games, which allows completing the minigame with selecting any single cell on the board."
	end
end